@@ -1,6 +1,6 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/docs/userguide/ch2a_fonts.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/docs/userguide/ch2a_fonts.py
 from tools.docco.rl_doc_utils import *
 from reportlab.lib.codecharts import SingleByteEncodingChart
 from reportlab.platypus import Image
@@ -49,7 +49,7 @@ includes converters for all the common encodings, including Asian ones.
 
 
 
-disc("""
+disc(u"""
 If your data is not encoded as UTF8, you will get a UnicodeDecodeError as
 soon as you feed in a non-ASCII character.  For example, this snippet below is
 attempting to read in and print a series of names, including one with a French
@@ -57,7 +57,7 @@ accent:  ^Marc-Andr\u00e9 Lemburg^.  The standard error is quite helpful and tel
 what character it doesn't like:
 """)
 
-eg("""
+eg(u"""
 >>> from reportlab.pdfgen.canvas import Canvas
 >>> c = Canvas('temp.pdf')
 >>> y = 700
@@ -216,7 +216,7 @@ for Type-1 fonts.
 disc("""
 Unfortunately, there is no reliable standard yet for such
 locations (not even on the same platform) and, hence, you might
-have to edit the file $reportlab/rl_config.py$ to modify the
+have to edit one of the files $reportlab_settings.py$ or $~/.reportlab_settings$ to modify the
 value of the $T1SearchPath$ identifier to contain additional
 directories.  Our own recommendation is to use the ^reportlab/fonts^
 folder in development; and to have any needed fonts as packaged parts of
@@ -402,7 +402,7 @@ pdfmetrics.registerFont(UnicodeCIDFont('HeiseiMin-W3'))
 canvas.setFont('HeiseiMin-W3', 16)
 
 # the two unicode characters below are "Tokyo"
-msg = u'\u6771\u4EAC : Unicode font, unicode input'
+msg = u'\\u6771\\u4EAC : Unicode font, unicode input'
 canvas.drawString(100, 675, msg)
 """)
 #had to double-escape the slashes above to get escapes into the PDF
