@@ -1,30 +1,32 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/frames.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/platypus/frames.py
 
-__version__='3.3.0'
+__version__='3.5.14'
 
 __doc__="""A frame is a container for content on a page.
 """
 
+__all__ = (
+            'ShowBoundaryValue',
+            'Frame',
+            )
+
 import logging
 logger = logging.getLogger('reportlab.platypus')
 
 _geomAttr=('x1', 'y1', 'width', 'height', 'leftPadding', 'bottomPadding', 'rightPadding', 'topPadding')
-from reportlab import rl_config, isPy3
+from reportlab import rl_config
 _FUZZ=rl_config._FUZZ
 
 class ShowBoundaryValue:
-    def __init__(self,color=(0,0,0),width=0.1):
+    def __init__(self,color=(0,0,0),width=0.1,dashArray=None):
         self.color = color
         self.width = width
+        self.dashArray = dashArray
 
-    if isPy3:
-        def __bool__(self):
-            return self.color is not None and self.width>=0
-    else:
-        def __nonzero__(self):
-            return self.color is not None and self.width>=0
+    def __bool__(self):
+        return self.color is not None and self.width>=0
 
 
 class Frame:
@@ -179,28 +181,42 @@ class Frame:
                 return 0
             else:
                 #now we can draw it, and update the current point.
-                s = flowable.getSpaceAfter()
+                sa = flowable.getSpaceAfter()
                 fbg = getattr(self,'_frameBGs',None)
-                if fbg:
-                    fbgl, fbgr, fbgc = fbg[-1]
+                if fbg and fbg[-1].active:
+                    bg = fbg[-1]
+                    fbgl = bg.left
+                    fbgr = bg.right
+                    bgm = bg.start
                     fbw = self._width-fbgl-fbgr
-                    fbh = y + h + s
-                    fby = max(p,y-s)
-                    fbh = max(0,fbh-fby)
-                    if abs(fbw)>_FUZZ and abs(fbh)>_FUZZ:
-                        canv.saveState()
-                        canv.setFillColor(fbgc)
-                        canv.rect(self._x1+fbgl,fby,fbw,fbh,stroke=0,fill=1)
-                        canv.restoreState()
+                    fbx = self._x1+fbgl
+                    if not bgm:
+                        fbh = y + h + sa
+                        fby = max(p,y-sa)
+                        fbh = max(0,fbh-fby)
+                    else:
+                        fbh = y + h - s
+                        att = fbh>=self._y2 - self._topPadding
+                        if bgm=='frame' or bgm=='frame-permanent' or (att and bgm=='frame-permanent-1'):
+                            #first time or att top use
+                            fbh = max(0,(self._y2 if att else fbh)-self._y1)
+                            fby = self._y1
+                            if bgm=='frame-permanent':
+                                fbg[-1].start = 'frame-permanent-1'
+                        else:
+                            fby = fbw = fbh = 0
+                    bg.render(canv,self,fbx,fby,fbw,fbh)
+                    if bgm=='frame':
+                        fbg.pop()
 
                 flowable.drawOn(canv, self._x + self._leftExtraIndent, y, _sW=aW-w)
                 flowable.canv=canv
                 if self._debug: logger.debug('drew %s' % flowable.identity())
-                y -= s
+                y -= sa
                 if self._oASpace:
                     if getattr(flowable,'_SPACETRANSFER',False):
-                        s = self._prevASpace
-                    self._prevASpace = s
+                        sa = self._prevASpace
+                    self._prevASpace = sa
                 if y!=self._y: self._atTop = 0
                 self._y = y
                 return 1
@@ -221,10 +237,13 @@ class Frame:
             s = flowable.getSpaceBefore()
             if self._oASpace:
                 s = max(s-self._prevASpace,0)
+        h = y-p-s
+        if h<=0 and not getattr(flowable,'_ZEROSIZE',False):
+            return []
         flowable._frame = self                  #some flowables might need these
-        flowable.canv = canv        
+        flowable.canv = canv
         try:
-            r = flowable.split(self._aW, y-p-s)
+            r = flowable.split(self._aW, h)
         finally:
             #sometimes canv/_frame aren't still on the flowable
             for a in ('canv', '_frame'):
@@ -233,32 +252,34 @@ class Frame:
         return r
 
 
-    def drawBoundary(self,canv):
+    @staticmethod
+    def _drawBoundary(canv,sb,x1,y1,width,height):
         "draw the frame boundary as a rectangle (primarily for debugging)."
         from reportlab.lib.colors import Color, toColor
-        sb = self.showBoundary
         ss = isinstance(sb,(str,tuple,list)) or isinstance(sb,Color)
         w = -1
+        da = None
         if ss:
-            c = toColor(sb,self)
-            ss = c is not self
+            c = toColor(sb,-1)
+            ss = c != -1
         elif isinstance(sb,ShowBoundaryValue) and sb:
-            c = toColor(sb.color,self)
-            w = sb.width
-            ss = c is not self
+            c = toColor(sb.color,-1)
+            ss = c != -1
+            if ss:
+                w = sb.width
+                da = sb.dashArray
         if ss:
             canv.saveState()
             canv.setStrokeColor(c)
-            if w>=0:
-                canv.setLineWidth(w)
-        canv.rect(
-                self._x1,
-                self._y1,
-                self._x2 - self._x1,
-                self._y2 - self._y1
-                )
+            if w>=0: canv.setLineWidth(w)
+            if da: canv.setDash(da)
+        canv.rect(x1,y1,width,height)
         if ss: canv.restoreState()
 
+    def drawBoundary(self,canv, __boundary__=None):
+        self._drawBoundary(canv,__boundary__ or self.showBoundary, self._x1, self._y1,
+                                self._x2 - self._x1, self._y2 - self._y1)
+
     def addFromList(self, drawlist, canv):
         """Consumes objects from the front of the list until the
         frame is full.  If it cannot fit one object, raises
