@@ -1,8 +1,5 @@
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
-#if PY_MAJOR_VERSION >= 3
-#	define isPy3
-#endif
 
 #ifndef PyMem_New
 	/*Niki Spahiev <niki@vintech.bg> suggests this is required for 1.5.2*/
@@ -12,6 +9,7 @@
 #include "libart_lgpl/libart.h"
 #include "gt1/gt1-parset1.h"
 #include "gt1/gt1-misc.h"
+#define FILL_UNSET -1
 #define FILL_EVEN_ODD 0
 #define FILL_NON_ZERO 1
 
@@ -21,28 +19,17 @@
 #endif
 
 
-#define VERSION "3.04"
+#define VERSION "4.03"
 #define MODULENAME "_renderPM"
-#ifdef isPy3
-#	define PyInt_FromLong	PyLong_FromLong
-#	define staticforward static
-#	define statichere static
+#define PyInt_FromLong	PyLong_FromLong
+#define staticforward static
+#define statichere static
 PyObject *RLPy_FindMethod(PyMethodDef *ml, PyObject *self, const char* name){
 	for(;ml->ml_name!=NULL;ml++)
 		if(name[0]==ml->ml_name[0] && strcmp(name+1,ml->ml_name+1)==0) return PyCFunction_New(ml, self);
 	return NULL;
 	}
 #define Py_FindMethod RLPy_FindMethod
-#else
-#   include "bytesobject.h"
-#	ifndef PyVarObject_HEAD_INIT
-#		define PyVarObject_HEAD_INIT(type, size) \
-        	PyObject_HEAD_INIT(type) size,
-#	endif
-#	ifndef Py_TYPE
-#		define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
-#	endif
-#endif
 #define __STR(x) #x
 #define STRINGIFY(x) __STR(x)
 #ifndef LIBART_VERSION
@@ -100,6 +87,7 @@ typedef	struct {
 		} gstateColorX;
 
 #ifdef	RENDERPM_FT
+#ifdef	RENDERPM_PARSE_UTF8
 static	PyObject* parse_utf8(PyObject* self, PyObject* args)
 {
 	char		*c, *msg;
@@ -141,6 +129,7 @@ RL_ERROR_EXIT:
     Py_INCREF(Py_None);
 	return Py_None;
 }
+#endif
 
 PyObject*	_pdfmetrics__fonts=0;
 static PyObject *_get_pdfmetrics__fonts(void){
@@ -187,11 +176,11 @@ static py_FT_FontObject *_get_ft_face(char *fontName)
 		}
 
 	ft_face = PyObject_NEW(py_FT_FontObject, &py_FT_Font_Type);
-	ft_face->face = NULL;
 	if(!ft_face){
 		PyErr_Format(PyExc_MemoryError, "Cannot allocate ft_face for TTFont %s", fontName);
 		goto RET;
 		}
+	ft_face->face = NULL;
 	face = PyObject_GetAttrString(font,"face");
 	if(!face) goto RET;
 	_data = PyObject_GetAttrString(face,"_ttf_data");
@@ -337,6 +326,7 @@ typedef struct {
 	double		strokeOpacity;
 	gstateColor	fillColor;			/*fill color*/
 	int			fillMode;
+	int			textRenderMode;
 	double		fillOpacity;
 	double		fontSize;
 	double		fontEMSize;			/*for scaling to points*/
@@ -617,23 +607,49 @@ static double _vpath_area(ArtVpath *p)
 	return a;
 }
 
-static	PyObject* gstate_clipPathSet(gstateObject* self, PyObject* args)
-{
+static	void _gstate_clipPathSetOrAdd(gstateObject* self, int fillMode, int add, int endIt){
 	ArtVpath	*vpath;
 	ArtVpath	*trVpath;
-	int			fillMode = self->fillMode;
-
-	if(!PyArg_ParseTuple(args,"|i:clipPathSet",&fillMode)) return NULL;
-	gstate_pathEnd(self);
+	if (fillMode==FILL_UNSET) fillMode = self->fillMode;
+	if(endIt) gstate_pathEnd(self);
 	dump_path(self);
 	vpath = art_bez_path_to_vec(self->path, VECSP);
 	dump_vpath("after -->vec",vpath);
 	trVpath = art_vpath_affine_transform (vpath, self->ctm);
 	_vpath_area(trVpath);
-	if(self->clipSVP) art_svp_free(self->clipSVP);
-	self->clipSVP = art_svp_from_vpath(trVpath);
+	if (add) {
+		ArtSVP		*newSVP = art_svp_from_vpath(trVpath);
+		if(self->clipSVP) {
+			ArtSVP *oldSVP = self->clipSVP;
+			self->clipSVP = art_svp_union(oldSVP,newSVP);
+			art_svp_free(oldSVP);
+			art_svp_free(newSVP);
+			}
+		else{
+			self->clipSVP = newSVP;
+			}
+		}
+	else{
+		if(self->clipSVP) art_svp_free(self->clipSVP);
+		self->clipSVP = art_svp_from_vpath(trVpath);
+		}
 	art_free(trVpath);
 	art_free(vpath);
+}
+
+static	PyObject* gstate_clipPathSet(gstateObject* self, PyObject* args)
+{
+	int			fillMode = FILL_UNSET;
+	if(!PyArg_ParseTuple(args,"|i:clipPathSet",&fillMode)) return NULL;
+	_gstate_clipPathSetOrAdd(self, fillMode, 0, 1); /*fill add endIt*/
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+static	PyObject* gstate_clipPathAdd(gstateObject* self, PyObject* args)
+{
+	int			fillMode = FILL_UNSET;
+	if(!PyArg_ParseTuple(args,"|i:clipPathAdd",&fillMode)) return NULL;
+	_gstate_clipPathSetOrAdd(self, fillMode, 1, 1) /*fill add endIt*/;
 	Py_INCREF(Py_None);
 	return Py_None;
 }
@@ -644,7 +660,6 @@ static void _gstate_pathFill(gstateObject* self,int endIt, int vpReverse, int fi
 	if(self->fillColor.valid){
 		ArtVpath	*vpath, *trVpath, *tmp_vpath;
 		ArtSVP		*svp, *tmp_svp;
-		ArtWindRule	wrule;
 		pixBufT*	p;
 		double		a;
 		if(endIt) gstate_pathEnd(self);
@@ -660,14 +675,13 @@ static void _gstate_pathFill(gstateObject* self,int endIt, int vpReverse, int fi
 			trVpath =  tmp_vpath;
 			svp = art_svp_from_vpath(trVpath);
 			dump_svp("fill svp from vpath",svp);
-			if(fillMode==0){
+			if(fillMode==FILL_EVEN_ODD){
 				tmp_svp = art_svp_uncross(svp);
 				dump_svp("fill svp uncrossed",tmp_svp);
-				wrule = fillMode==FILL_EVEN_ODD ? ART_WIND_RULE_ODDEVEN : ART_WIND_RULE_NONZERO;
 				art_svp_free(svp);
-				svp = art_svp_rewind_uncrossed(tmp_svp,wrule);
+				svp = art_svp_rewind_uncrossed(tmp_svp,fillMode==FILL_EVEN_ODD ? ART_WIND_RULE_ODDEVEN : ART_WIND_RULE_NONZERO);
 				art_svp_free(tmp_svp);
-				}
+			}
 			if(self->clipSVP) {
 				tmp_svp = svp;
 				dump_svp("fill clip svp path",self->clipSVP);
@@ -702,21 +716,20 @@ static PyObject* gstate_pathFill(gstateObject* self, PyObject* args)
 {
 	int			fillMode = self->fillMode;
 	if(!PyArg_ParseTuple(args,"|i:pathFill",&fillMode)) return NULL;
-	_gstate_pathFill(self,1,0,fillMode);
+	_gstate_pathFill(self,1,0,fillMode); /*endIt vpReverse(ignored) fillMode*/
 	Py_INCREF(Py_None);
 	return Py_None;
 }
 
-static PyObject* gstate_pathStroke(gstateObject* self, PyObject* args)
+static void _gstate_pathStroke(gstateObject* self, int endIt)
 {
 	ArtVpath	*vpath=NULL, *trVpath;
 	ArtSVP*		svp=NULL;
 	ArtSVP*		tmp_svp=NULL;
 	pixBufT*	p;
 
-	if(!PyArg_ParseTuple(args,":pathStroke")) return NULL;
 	if(self->strokeColor.valid && self->strokeWidth>0){
-		gstate_pathEnd(self);
+		if(endIt) gstate_pathEnd(self);
 		dump_path(self);
 		vpath = art_bez_path_to_vec(self->path, VECSP);
 
@@ -750,6 +763,12 @@ static PyObject* gstate_pathStroke(gstateObject* self, PyObject* args)
 		art_svp_free(svp);
 		art_free(vpath);
 		}
+}
+
+static PyObject* gstate_pathStroke(gstateObject* self, PyObject* args)
+{
+	if(!PyArg_ParseTuple(args,":pathStroke")) return NULL;
+	_gstate_pathStroke(self, 1);	/*endIt*/
 	Py_INCREF(Py_None);
 	return Py_None;
 }
@@ -847,7 +866,7 @@ static PyObject* gstate_drawString(gstateObject* self, PyObject* args)
 	A2DMX	orig, trans = {1,0,0,1,0,0}, scaleMat = {1,0,0,1,0,0};
 	double	scaleFactor, x, y, w;
 	char*	text;
-	int		c, i;
+	int		c, i, textRenderMode=self->textRenderMode;
 	Py_ssize_t	textlen;
 	ArtBpath	*saved_path, *path;
 	void	*font = self->font;
@@ -941,7 +960,13 @@ static PyObject* gstate_drawString(gstateObject* self, PyObject* args)
 
 		if(path){
 			self->path = path;
-			_gstate_pathFill(self,0,1,1);
+			if (textRenderMode==0 || textRenderMode==2 || textRenderMode==4 || textRenderMode==6)
+				_gstate_pathFill(self,0,1,FILL_NON_ZERO) /*endit vpReverse(ignored) fillMode*/;
+			if (textRenderMode==1 || textRenderMode==2 || textRenderMode==5 || textRenderMode==6)
+				_gstate_pathStroke(self,0);	/*endIt*/
+			if (textRenderMode>=4)
+				_gstate_clipPathSetOrAdd(self, -1, 1, 0);	/*fill add endIt*/
+
 #ifdef	RENDERPM_FT
 			if(!ft_font && path!=notdefPath)
 #endif
@@ -1190,7 +1215,10 @@ static PyObject* gstate_setFont(gstateObject* self, PyObject* args)
 	if(!PyArg_ParseTuple(args,"Od:setFont", &fontNameObj, &fontSize)) return NULL;
 	if(PyUnicode_Check(fontNameObj)){
 		b=PyUnicode_AsUTF8String(fontNameObj);
-		if(!b) goto err;
+		if(!b){
+			PyErr_SetString(PyExc_ValueError, "_renderPM.gstate_setFont: bytes conversion of fontName failed");
+			goto err;
+			}
 		fontName = PyBytes_AsString(b);
 		}
 	else{
@@ -1209,7 +1237,7 @@ static PyObject* gstate_setFont(gstateObject* self, PyObject* args)
 		fontEMSize = 1000.;
 		ft_font = 0;
 		}
-#ifdef	RENDERPM_FT
+#ifdef RENDERPM_FT
 	else{
 		f = (Gt1EncodedFont*)_ft_get_face(fontName);
 		fontEMSize = f ? ((FT_Face)f)->units_per_EM : 0;
@@ -1224,7 +1252,7 @@ static PyObject* gstate_setFont(gstateObject* self, PyObject* args)
 		self->fontNameObj = fontNameObj;
 		Py_INCREF(fontNameObj);
 		self->fontEMSize = fontEMSize;
-#ifdef	RENDERPM_FT
+#ifdef RENDERPM_FT
 		self->ft_font = ft_font;
 #endif
 		Py_INCREF(Py_None);
@@ -1298,11 +1326,7 @@ static PyObject* gstate__aapixbuf(gstateObject* self, PyObject* args)
 	ArtPixBuf	src;
 
 	src.n_channels = 3;
-#ifdef isPy3
-	#define AAPIXBUFFMT "ddddy#ii|i:_aapixbuf"
-#else
-	#define AAPIXBUFFMT "ddddt#ii|i:_aapixbuf"
-#endif
+#define AAPIXBUFFMT "ddddy#ii|i:_aapixbuf"
 
 	/*(dstX,dstY,dstW,dstH,src,srcW,srcH[,srcD[,aff]])*/
 	if(!PyArg_ParseTuple(args,AAPIXBUFFMT,
@@ -1511,6 +1535,7 @@ static PyObject* _get_gstateFontName(gstateObject *self)
 static struct PyMethodDef gstate_methods[] = {
 	{"clipPathClear", (PyCFunction)gstate_clipPathClear, METH_VARARGS, "clipPathClear()"},
 	{"clipPathSet", (PyCFunction)gstate_clipPathSet, METH_VARARGS, "clipPathSet([fillMode])"},
+	{"clipPathAdd", (PyCFunction)gstate_clipPathAdd, METH_VARARGS, "clipPathAdd([fillMode])"},
 	{"curveTo", (PyCFunction)gstate_curveTo, METH_VARARGS, "curveTo(x1,y1,x2,y2,x3,y3)"},
 	{"drawString", (PyCFunction)gstate_drawString, METH_VARARGS, "drawString(x,y,text)"},
 	{"lineTo", (PyCFunction)gstate_lineTo, METH_VARARGS, "lineTo(x,y)"},
@@ -1551,6 +1576,7 @@ static PyObject* gstate_getattr(gstateObject *self, char *name)
 	else if(!strcmp(name,"fontName")) return _get_gstateFontName(self);
 	else if(!strcmp(name,"fontNameI")) return _get_gstateFontNameI(self);
 	else if(!strcmp(name,"dashArray")) return _get_gstateDashArray(self);
+	else if(!strcmp(name,"textRenderMode")) return PyInt_FromLong(self->textRenderMode);
 	else if(!strcmp(name,"pixBuf")){
 		pixBufT* p = self->pixBuf;
 		int	nw = p->rowstride;
@@ -1575,7 +1601,7 @@ static PyObject* gstate_getattr(gstateObject *self, char *name)
 
 static int gstate_setattr(gstateObject *self, char *name, PyObject* value)
 {
-	int	i;
+	int	i, tmp;
 #ifdef	ROBIN_DEBUG
 	printf("setattr(%s)\n", name);
 #endif
@@ -1589,6 +1615,16 @@ static int gstate_setattr(gstateObject *self, char *name, PyObject* value)
 	else if(!strcmp(name,"strokeOpacity")) i = PyArg_Parse(value,"d",&self->strokeOpacity);
 	else if(!strcmp(name,"fillOpacity")) i = PyArg_Parse(value,"d",&self->fillOpacity);
 	else if(!strcmp(name,"dashArray")) i = _set_gstateDashArray(value,self);
+	else if(!strcmp(name,"textRenderMode")) i = PyArg_Parse(value,"i",&self->textRenderMode);
+	else if(!strcmp(name,"fontSize")) {
+		i = PyArg_Parse(value,"i",&tmp);
+		if(!i) {
+			if(tmp<0) {
+				i = -1;
+				PyErr_SetString(PyExc_ValueError, name);
+				}
+			}
+		}
 	else {
 		PyErr_SetString(PyExc_AttributeError, name);
 		i = 0;
@@ -1621,6 +1657,7 @@ static	void gstateFree(gstateObject* self)
 gstates have the following methods\n\
  clipPathClear() clear clipPath\n\
  clipPathSet([fillMode]) move current path into clipPath\n\
+ clipPathAdd([fillMode]) add current path into clipPath\n\
  curveTo(x1,y1,x2,y2,x3,y3)  #add a curveTo type segment\n\
  drawString(x,y,text)\n\
  moveTo(x,y) start a segment\n\
@@ -1641,6 +1678,7 @@ ctm			6vec float transformation matrix\n\
 strokeColor 32bit stroke colour\n\
 fillColor	32bit fill colour\n\
 fillMode	int fill rule\n\
+textRenderMode	int controls filling and stroking (follows pdf values)\n\
 lineCap		int\n\
 lineJoin	int\n\
 hasClipPath readonly int\n\
@@ -1740,6 +1778,7 @@ static	gstateObject* gstate(PyObject* module, PyObject* args, PyObject* keywds)
 			self->strokeColor.valid = self->fillColor.valid = 0;
 			self->lineCap = self->lineJoin = 0;
 			self->fillMode = FILL_NON_ZERO;	/*our default for renderPM differed from PDF/PS*/
+			self->textRenderMode = 0;
 			self->strokeOpacity = self->strokeWidth = self->fillOpacity = 1.0;
 			self->pathLen = 0;
 			self->pathMax = m;
@@ -1938,7 +1977,7 @@ static int pict_putRow(BYTE_STREAM* fd, int row, int cols, pixel* rowpixels, cha
 	register char *p;
 
 	run = count = 0;
-	for (cols--, i = cols, pP = rowpixels + cols, p = packed, lastp = *pP;
+	for (i = cols-1, pP = rowpixels + i, p = packed, lastp = *pP;
 		i >= 0; i--, lastp = *pP, pP--){
 		if (lastp == *pP) run++;
 		else if (run < RUN_THRESH){
@@ -1983,14 +2022,14 @@ static int pict_putRow(BYTE_STREAM* fd, int row, int cols, pixel* rowpixels, cha
 			rep = run > MAX_RUN ? MAX_RUN : run;
 			*p++ = lastp;
 			*p++ = runtochar(rep);
-				run -= rep;
+			run -= rep;
 			}
 		run = 1;
 		}
 	if (count > 0) *p++ = counttochar(count);
 
 	packcols = (int)(p - packed);		/* how many did we write? */
-	if (cols > 250){
+	if (cols >= 250){/*bytesperline>=250*/
 		pict_putShort(fd, packcols);
 		oc = packcols + 2;
 		}
@@ -2121,7 +2160,9 @@ static struct PyMethodDef _methods[] = {
 	{"pil2pict", (PyCFunction)pil2pict, METH_VARARGS, "pil2pict(cols,rows,datastr,palette) return PICT version of im as bytes"},
 #ifdef	RENDERPM_FT
     {"ft_get_face", (PyCFunction)ft_get_face, METH_VARARGS|METH_KEYWORDS,"ft_get_face(fontName) --> ft_face instance"},
+#ifdef	RENDERPM_PARSE_UTF8
 	{"parse_utf8", (PyCFunction)parse_utf8, METH_VARARGS, "parse_utf8(utf8_string) return UCS list"},
+#endif
 #endif /*ifdef	RENDERPM_FT*/
 #ifdef MEMORY_DEBUG
     {"mtrace", (PyCFunction)_py_mtrace, METH_VARARGS|METH_KEYWORDS,"mtrace(startStop) start or stop malloc tracing"},
@@ -2134,7 +2175,6 @@ extern void (*libart_set_DYNAMIC_BIGENDIAN(void));
 #endif
 
 /*Initialization function for the module*/
-#ifdef isPy3
 static struct PyModuleDef moduleDef = {
 	PyModuleDef_HEAD_INIT,
 	MODULENAME,
@@ -2148,15 +2188,6 @@ static struct PyModuleDef moduleDef = {
 	};
 
 PyMODINIT_FUNC PyInit__renderPM(void)
-#define OK_RET m
-#define ERR_RET NULL
-#define CREATE_MODULE() PyModule_Create(&moduleDef)
-#else
-void init_renderPM(void)
-#define OK_RET
-#define ERR_RET
-#define CREATE_MODULE() Py_InitModule(MODULENAME, _methods)
-#endif
 {
 	PyObject *m=NULL, *obj=NULL;
 
@@ -2167,7 +2198,7 @@ void init_renderPM(void)
 #endif
 
 	/* Create the module and add the functions */
-	m = CREATE_MODULE();
+	m = PyModule_Create(&moduleDef);
     if(!m)goto err;
 
 	/* Add some symbolic constants to the module */
@@ -2183,17 +2214,12 @@ void init_renderPM(void)
 	obj = PyUnicode_FromString(__FILE__);
 	if(!obj)goto err;
 	PyModule_AddObject(m, "__file__", obj);
-#ifndef isPy3
-	obj = PyUnicode_FromString(__DOC__);
-	if(!obj)goto err;
-	PyModule_AddObject(m, "__doc__", obj);/*add in the docstring*/
-#endif
 #ifdef DYNAMIC_ENDIANNESS
 	libart_set_DYNAMIC_BIGENDIAN();
 #endif
-	return OK_RET;
+	return m;
 err:
 	Py_XDECREF(obj);
 	Py_XDECREF(m);
-	return ERR_RET;
+	return NULL;
 }
