@@ -7,9 +7,6 @@
 */
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
-#if PY_MAJOR_VERSION >= 3
-#	define isPy3
-#endif
 #include <stdlib.h>
 #include <math.h>
 #define DEFERRED_ADDRESS(A) 0
@@ -30,75 +27,89 @@
 #ifndef min
 #	define min(a,b) ((a)<(b)?(a):(b))
 #endif
-#define VERSION "0.80"
+#define VERSION "0.81"
 #define MODULE "_rl_accel"
 
-struct module_state	{
-	int moduleLineno;
-#ifndef isPy3
-	PyObject *module;
-#endif
-	};
-#ifdef isPy3
-#	define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
-#	define STRNAME "str"
-#	define BYTESNAME "bytes"
-#else
-	static struct module_state _state;
-#	include "bytesobject.h"
-#	ifndef PyVarObject_HEAD_INIT
-#		define PyVarObject_HEAD_INIT(type, size) \
-        	PyObject_HEAD_INIT(type) size,
-#	endif
-#	ifndef Py_TYPE
-#		define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
-#	endif
-#	define PyBytes_AS_STRING	PyString_AS_STRING
-#	define PyBytes_AsString		PyString_AsString
-#	define PyBytes_GET_SIZE 	PyString_GET_SIZE
-#	define GETSTATE(m) (&_state)
-#	define STRNAME "unicode"
-#	define BYTESNAME "str"
-#endif
-
-#define ERROR_EXIT() {GETSTATE(module)->moduleLineno=__LINE__;goto L_ERR;}
-#define ADD_TB(module,name) _add_TB(module,name)
-#include "compile.h"
-#include "frameobject.h"
-#include "traceback.h"
-static void _add_TB(PyObject *module,char *funcname)
+#define STRNAME "str"
+#define BYTESNAME "bytes"
+static void ModifyExcValue(PyObject *exc,const char *funcname,int lineno,const char* fmt,va_list ap)
 {
-	int	moduleLineno = GETSTATE(module)->moduleLineno;
-	PyObject *py_globals = NULL;
-	PyCodeObject *py_code = NULL;
-	PyFrameObject *py_frame = NULL;
+	PyObject *type = NULL, *value = NULL, *tb = NULL, *aval=NULL, *uval=NULL;
+	const char* sval=NULL;
+	PyErr_Fetch(&type, &value, &tb);
+	PyErr_NormalizeException(&type, &value, &tb);
+	if(PyErr_Occurred()) goto L_BAD;
+	uval = PyUnicode_FromFormatV(fmt,ap);
+	if(uval){
+		PyErr_Format(exc,"%U in %s @ %s:%d\ncaused by %S",uval,funcname,__FILE__,lineno,value);
+		}
+	else {
+		PyErr_Format(exc,"in %s:%d\ncaused by %S",funcname,__FILE__,lineno,value);
+		}
 
-#ifdef isPy3
-	py_globals = PyModule_GetDict(module);
-#else
-	py_globals = PyModule_GetDict(GETSTATE(module)->module);
-#endif
-	if(!py_globals) goto bad;
-	py_code = PyCode_NewEmpty(
-						__FILE__,		/*PyObject *filename,*/
-						funcname,	/*PyObject *name,*/
-						moduleLineno	/*int firstlineno,*/
-						);
-	if(!py_code) goto bad;
-	py_frame = PyFrame_New(
-		PyThreadState_Get(), /*PyThreadState *tstate,*/
-		py_code,			 /*PyCodeObject *code,*/
-		py_globals,			 /*PyObject *globals,*/
-		0					 /*PyObject *locals*/
-		);
-	if(!py_frame) goto bad;
-	py_frame->f_lineno = moduleLineno;
-	PyTraceBack_Here(py_frame);
-bad:
-	Py_XDECREF(py_code);
-	Py_XDECREF(py_frame);
+L_exit:
+	Py_XDECREF(uval);
+	Py_XDECREF(type);
+	Py_XDECREF(value);
+	Py_XDECREF(tb);
+	return;
+L_BAD:
+	if(type && value){
+		PyErr_Restore(type,value,tb);
+		type = value = tb = NULL;
+		}
+	goto L_exit;
 }
 
+static void _excAddInfo(const char* funcname,int lineno, PyObject *exc, const char* fmt, va_list ap)
+{
+	PyObject *uval = NULL;
+	if(PyErr_Occurred()){
+		ModifyExcValue(exc,funcname,lineno,fmt,ap);
+		}
+	else{
+		uval = PyUnicode_FromFormatV(fmt,ap);
+		if(uval) {
+			PyErr_Format(exc,"in %s@%s:%d %U",funcname,__FILE__,lineno,uval);
+			Py_DECREF(uval);
+			}
+		else
+			PyErr_Format(exc,"in %s@%s:%d",funcname,__FILE__,lineno,uval);
+		}
+}
+
+static void excAddInfo(const char* funcname,int lineno, PyObject *exc, const char* fmt, ...)
+{
+	va_list ap;
+	va_start(ap,fmt);
+	_excAddInfo(funcname,lineno,exc,fmt,ap);
+	va_end(ap);
+}
+/*
+ * https://stackoverflow.com/questions/5588855/standard-alternative-to-gccs-va-args-trick
+ * expands to the first argument
+ */
+#define FIRST(...) FIRST_HELPER(__VA_ARGS__, throwaway)
+#define FIRST_HELPER(first, ...) first
+/*
+ * if there's only one argument, expands to nothing.  if there is more
+ * than one argument, expands to a comma followed by everything but
+ * the first argument.  only supports up to 9 arguments but can be
+ * trivially expanded.
+ */
+#define REST(...) REST_HELPER(NUM(__VA_ARGS__), __VA_ARGS__)
+#define REST_HELPER(qty, ...) REST_HELPER2(qty, __VA_ARGS__)
+#define REST_HELPER2(qty, ...) REST_HELPER_##qty(__VA_ARGS__)
+#define REST_HELPER_ONE(first)
+#define REST_HELPER_TWOORMORE(first, ...) , __VA_ARGS__
+#define NUM(...) \
+    SELECT_10TH(__VA_ARGS__, TWOORMORE, TWOORMORE, TWOORMORE, TWOORMORE,\
+                TWOORMORE, TWOORMORE, TWOORMORE, TWOORMORE, ONE, throwaway)
+#define SELECT_10TH(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, ...) a10
+#define EXC_SET(exc,...) do{excAddInfo(__func__,__LINE__,exc,FIRST(__VA_ARGS__) REST(__VA_ARGS__));}while(0)
+#define EXC_EXIT(exc,...)  do{EXC_SET(exc,FIRST(__VA_ARGS__) REST(__VA_ARGS__));goto L_exit;}while(0)
+#define MODULE_STATE_SIZE 0
+
 #define a85_0		   1L
 #define a85_1		   85L
 #define a85_2		 7225L
@@ -117,19 +128,16 @@ PyObject *_a85_encode(PyObject *module, PyObject *args)
 	if(PyUnicode_Check(inObj)){
 		_o1 = PyUnicode_AsLatin1String(inObj);
 		if(!_o1){
-			PyErr_SetString(PyExc_ValueError,"argument not decodable as latin1");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not decodable as latin1");
 			}
 		inData = PyBytes_AsString(_o1);
 		inObj = _o1;
 		if(!inData){
-			PyErr_SetString(PyExc_ValueError,"argument not converted to internal char string");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not converted to internal char string");
 			}
 		}
 	else if(!PyBytes_Check(inObj)){
-		PyErr_SetString(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
 		}
 	inData = PyBytes_AsString(inObj);
 	length = PyBytes_GET_SIZE(inObj);
@@ -196,15 +204,11 @@ PyObject *_a85_encode(PyObject *module, PyObject *args)
 	retVal = PyUnicode_FromStringAndSize(buf, k);
 	free(buf);
 	if(!retVal){
-		PyErr_SetString(PyExc_ValueError,"failed to create return " STRNAME " value" );
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError,"failed to create return " STRNAME " value" );
 		}
 L_exit:
 	Py_XDECREF(_o1);
 	return retVal;
-L_ERR:
-	ADD_TB(module,"asciiBase85Encode");
-	goto L_exit;
 }
 
 PyObject *_a85_decode(PyObject *module, PyObject *args)
@@ -217,19 +221,16 @@ PyObject *_a85_decode(PyObject *module, PyObject *args)
 	if(PyUnicode_Check(inObj)){
 		_o1 = PyUnicode_AsLatin1String(inObj);
 		if(!_o1){
-			PyErr_SetString(PyExc_ValueError,"argument not decodable as latin1");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not decodable as latin1");
 			}
 		inData = PyBytes_AsString(_o1);
 		inObj = _o1;
 		if(!inData){
-			PyErr_SetString(PyExc_ValueError,"argument not converted to internal char string");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not converted to internal char string");
 			}
 		}
 	else if(!PyBytes_Check(inObj)){
-		PyErr_SetString(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
 		}
 	inData = PyBytes_AsString(inObj);
 	length = (unsigned int)PyBytes_GET_SIZE(inObj);
@@ -250,9 +251,8 @@ PyObject *_a85_decode(PyObject *module, PyObject *args)
 	length = (unsigned int)(q - inData);
 	buf = inData+length-2;
 	if(buf[0]!='~' || buf[1]!='>'){
-		PyErr_SetString(PyExc_ValueError, "Invalid terminator for Ascii Base 85 Stream");
 		free(inData);
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError, "Invalid terminator for Ascii Base 85 Stream");
 		}
 	length -= 2;
 	buf[0] = 0;
@@ -295,15 +295,11 @@ PyObject *_a85_decode(PyObject *module, PyObject *args)
 	free(buf);
 	free(tmp);
 	if(!retVal){
-		PyErr_SetString(PyExc_ValueError,"failed to create return " BYTESNAME " value" );
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError,"failed to create return " BYTESNAME " value" );
 		}
 L_exit:
 	Py_XDECREF(_o1);
 	return retVal;
-L_ERR:
-	ADD_TB(module,"asciiBase85Decode");
-	goto L_exit;
 }
 
 static	char* _fp_fmts[]={"%.0f", "%.1f", "%.2f", "%.3f", "%.4f", "%.5f", "%.6f"};
@@ -313,12 +309,13 @@ static	char *_fp_one(PyObject* module,PyObject *pD)
 	static	char s[30];
 	int l;
 	char*	dot;
-	if((pD=PyNumber_Float(pD))){
-		d = PyFloat_AS_DOUBLE(pD);
-		Py_DECREF(pD);
+	PyObject *cD;
+	if((cD=PyNumber_Float(pD))){
+		d = PyFloat_AS_DOUBLE(cD);
+		Py_DECREF(cD);
 		}
 	else {
-		PyErr_SetString(PyExc_ValueError, "bad numeric value");
+		EXC_SET(PyExc_ValueError, "bad numeric value %S",pD);
 		return NULL;
 		}
 	ad = fabs(d);
@@ -328,7 +325,7 @@ static	char *_fp_one(PyObject* module,PyObject *pD)
 		}
 	else{
 		if(ad>1e20){
-			PyErr_SetString(PyExc_ValueError, "number too large");
+			EXC_SET(PyExc_ValueError, "number too large %S",pD);
 			return NULL;
 			}
 		if(ad>1) l = min(max(0,6-(int)log10(ad)),6);
@@ -379,7 +376,7 @@ PyObject *_fp_str(PyObject *module, PyObject *args)
 			else pD = NULL;
 			if(!pD){
 				free(buf);
-				return NULL;
+				EXC_EXIT(PyExc_ValueError,"_fp_one failed");
 				}
 			if(pB!=buf){
 				*pB++ = ' ';
@@ -388,18 +385,14 @@ PyObject *_fp_str(PyObject *module, PyObject *args)
 			pB = pB + strlen(pB);
 			}
 		*pB = 0;
-#ifdef isPy3
 		retVal = PyUnicode_FromString(buf);
-#else
-		retVal = PyBytes_FromString(buf);
-#endif
 		free(buf);
 		return retVal;
 		}
 	else {
 		PyErr_Clear();
 		PyArg_ParseTuple(args, "O:_fp_str", &retVal);
-		return NULL;
+L_exit: return NULL;
 		}
 }
 
@@ -424,11 +417,7 @@ static PyObject *_escapePDF(unsigned char* text, Py_ssize_t textlen)
 			out[j++] = c;
 			}
 		}
-#ifdef isPy3
 	ret = PyUnicode_FromStringAndSize((const char *)out,j);
-#else
-	ret = PyBytes_FromStringAndSize((const char *)out,j);
-#endif
 	PyMem_Free(out);
 	return ret;
 }
@@ -443,19 +432,16 @@ static PyObject *escapePDF(PyObject *module, PyObject* args)
 	if(PyUnicode_Check(inObj)){
 		_o1 = PyUnicode_AsLatin1String(inObj);
 		if(!_o1){
-			PyErr_SetString(PyExc_ValueError,"argument not decodable as latin1");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not decodable as latin1");
 			}
 		inData = PyBytes_AsString(_o1);
 		inObj = _o1;
 		if(!inData){
-			PyErr_SetString(PyExc_ValueError,"argument not converted to internal char string");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError,"argument not converted to internal char string");
 			}
 		}
 	else if(!PyBytes_Check(inObj)){
-		PyErr_SetString(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
-		ERROR_EXIT();
+		EXC_EXIT(PyExc_ValueError,"argument should be " BYTESNAME " or latin1 decodable " STRNAME);
 		}
 	inData = PyBytes_AsString(inObj);
 	length = PyBytes_GET_SIZE(inObj);
@@ -463,9 +449,6 @@ static PyObject *escapePDF(PyObject *module, PyObject* args)
 L_exit:
 	Py_XDECREF(_o1);
 	return retVal;
-L_ERR:
-	ADD_TB(module,"excapePDF");
-	goto L_exit;
 }
 
 static PyObject *sameFrag(PyObject *module, PyObject* args)
@@ -482,11 +465,7 @@ static PyObject *sameFrag(PyObject *module, PyObject* args)
 		fa = PyObject_GetAttrString(f,*p);
 		ga = PyObject_GetAttrString(g,*p);
 		if(fa && ga){
-#ifdef	isPy3
 			t = PyObject_RichCompareBool(fa,ga,Py_NE);
-#else
-			t = PyObject_Compare(fa,ga);
-#endif
 			Py_DECREF(fa);
 			Py_DECREF(ga);
 			if(PyErr_Occurred()) goto L1;
@@ -575,7 +554,7 @@ L_BAD:
 static PyObject *_GetAttrString(PyObject *obj, char *name)
 {
 	PyObject *res = PyObject_GetAttrString(obj, name);
-	if(!res) PyErr_SetString(PyExc_AttributeError, name);
+	if(!res) EXC_SET(PyExc_AttributeError, "missing attribute %s",name);
 	return res;
 }
 
@@ -587,19 +566,9 @@ static PyObject *_GetStringBuf(PyObject *obj, const char **buf)
 	PyObject *res;
 
 	if(PyUnicode_Check(obj)){
-#ifdef	isPy3
 		res = obj;
 		Py_INCREF(res);
 		*buf = (const char *)PyUnicode_AsUTF8(res);
-#else
-		res = PyUnicode_AsUTF8String(obj);
-		if (!res) {
-			PyErr_SetString(PyExc_ValueError,
-					"encode to UTF8 bytes failed");
-			return NULL;
-		}
-		*buf = PyBytes_AsString(res);
-#endif
 		}
 	else if(PyBytes_Check(obj)){
 		res = obj;
@@ -607,8 +576,7 @@ static PyObject *_GetStringBuf(PyObject *obj, const char **buf)
 		*buf = PyBytes_AsString(res);
 		}
 	else{
-		PyErr_SetString(PyExc_ValueError,
-				"require bytes or unicode object");
+		EXC_SET(PyExc_ValueError, "require bytes or unicode object");
 		return NULL;
 		}
 	return res;
@@ -630,116 +598,115 @@ static PyObject *unicode2T1(PyObject *module, PyObject *args, PyObject *kwds)
 	font = Py_None; Py_INCREF(Py_None);
 
 
-	_o2 = PyList_New(0); if(!_o2) ERROR_EXIT();
+	_o2 = PyList_New(0); if(!_o2) EXC_EXIT(PyExc_MemoryError,"unable to create list");
 	Py_DECREF(R);
 	R = _o2;
 	_o2 = NULL;
 
-	_o2 = PySequence_GetItem(fonts,0); if(!_o2) ERROR_EXIT();
-	_o1 = PySequence_GetSlice(fonts, 1, 0x7fffffff); if(!_o1) ERROR_EXIT();
+	_o2 = PySequence_GetItem(fonts,0); if(!_o2) EXC_EXIT(PyExc_IndexError,"fonts[0] failed");
+	_o1 = PySequence_GetSlice(fonts, 1, 0x7fffffff); if(!_o1) EXC_EXIT(PyExc_IndexError,"fonts[1:] failed");
 	Py_DECREF(font);
 	font = _o2;
 	Py_DECREF(fonts);
 	fonts = _o1;
 	_o1 = _o2 = NULL;
 
-	_o2 = _GetAttrString(font, "encName"); if(!_o2) ERROR_EXIT();
+	_o2 = _GetAttrString(font, "encName"); if(!_o2) EXC_EXIT(PyExc_AttributeError,"no encName");
 	encObj = _GetStringBuf(_o2, &encStr);
 	Py_DECREF(_o2);
 	_o2 = NULL;
-	if (!encObj) ERROR_EXIT();
+	if (!encObj) EXC_EXIT(PyExc_TypeError,"could not convert str(font.encName) failed");
 	if(strstr(encStr,"UCS-2")) encStr = "UTF16";
 
 	while((_i1=PyObject_IsTrue(utext))>0){
 		if((_o1 = PyUnicode_AsEncodedString(utext, encStr, NULL))){
-			_o2 = PyTuple_New(2); if(!_o2) ERROR_EXIT();
+			_o2 = PyTuple_New(2); if(!_o2) EXC_EXIT(PyExc_MemoryError,"create tuple length 2 failed");
 			Py_INCREF(font);
 			PyTuple_SET_ITEM(_o2, 0, font);
 			PyTuple_SET_ITEM(_o2, 1, _o1);
 			_o1 = NULL;
-			if(PyList_Append(R, _o2)) ERROR_EXIT();
+			if(PyList_Append(R, _o2)) EXC_EXIT(PyExc_RuntimeError,"could not append to result list");
 			Py_DECREF(_o2); _o2 = NULL;
 			break;
 			}
 		else{
 			Py_XDECREF(_o1); _o1 = NULL;
-			if(!PyErr_ExceptionMatches(PyExc_UnicodeEncodeError)) ERROR_EXIT();
-			_o1 = _GetExcValue(); if(!_o1) ERROR_EXIT();
+			if(!PyErr_ExceptionMatches(PyExc_UnicodeEncodeError)) EXC_EXIT(PyExc_RuntimeError,"unexpected exception");
+			_o1 = _GetExcValue(); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"could not obtain exception value");
 			PyErr_Clear();
-			_o2 = _GetAttrString(_o1, "args"); if(!_o2) ERROR_EXIT();
+			_o2 = _GetAttrString(_o1, "args"); if(!_o2) EXC_EXIT(PyExc_AttributeError,"missing args attribute");
 			Py_DECREF(_o1);
-			_o1 = PySequence_GetSlice(_o2, 2, 4); if(!_o1) ERROR_EXIT();
+			_o1 = PySequence_GetSlice(_o2, 2, 4); if(!_o1) EXC_EXIT(PyExc_IndexError,"args[2:4] failed");
 			Py_DECREF(_o2);
-			_o2 = PySequence_GetItem(_o1, 0); if(!_o2) ERROR_EXIT();
-			i = PyLong_AsLong(_o2); if(PyErr_Occurred()) ERROR_EXIT();
+			_o2 = PySequence_GetItem(_o1, 0); if(!_o2) EXC_EXIT(PyExc_IndexError,"args[2:4][0] failed");
+			i = PyLong_AsLong(_o2); if(PyErr_Occurred()) EXC_EXIT(PyExc_ValueError,"int(args[2:4][0]) failed");
 			Py_DECREF(_o2);
 
-			_o2 = PySequence_GetItem(_o1, 1); if(!_o1) ERROR_EXIT();
-			j = PyLong_AsLong(_o2); if(PyErr_Occurred()) ERROR_EXIT();
+			_o2 = PySequence_GetItem(_o1, 1); if(!_o1) EXC_EXIT(PyExc_IndexError,"args[2:4][1] failed");
+			j = PyLong_AsLong(_o2); if(PyErr_Occurred()) EXC_EXIT(PyExc_ValueError,"int(args[2:4][1]) failed");
 			Py_DECREF(_o2);
 
 			Py_DECREF(_o1); _o2 = _o1 = 0;
 
 			if(i){
-				_o1 = PySequence_GetSlice(utext, 0, i); if(!_o1) ERROR_EXIT();
-				_o2 = PyUnicode_AsEncodedString(_o1, encStr, NULL); if(!_o2) ERROR_EXIT();
+				_o1 = PySequence_GetSlice(utext, 0, i); if(!_o1) EXC_EXIT(PyExc_IndexError,"utext[0:%d] failed",i);
+				_o2 = PyUnicode_AsEncodedString(_o1, encStr, NULL); if(!_o2) EXC_EXIT(PyExc_UnicodeEncodeError,"encode(utext[0:%d],'%s') failed",i,encStr);
 				Py_DECREF(_o1);
-				_o1 = PyTuple_New(2); if(!_o1) ERROR_EXIT();
+				_o1 = PyTuple_New(2); if(!_o1) EXC_EXIT(PyExc_MemoryError,"create tuple of length 2 failed");
 				Py_INCREF(font);
 				PyTuple_SET_ITEM(_o1, 0, font);
 				PyTuple_SET_ITEM(_o1, 1, _o2);
 				_o2 = NULL;
-				if(PyList_Append(R, _o1)) ERROR_EXIT();
+				if(PyList_Append(R, _o1)) EXC_EXIT(PyExc_RuntimeError,"could not append to result list");
 				Py_DECREF(_o1); _o1 = NULL;
 				}
 
-			_i2 = PyObject_IsTrue(fonts); if(_i2<0) ERROR_EXIT();
+			_i2 = PyObject_IsTrue(fonts); if(_i2<0) EXC_EXIT(PyExc_ValueError,"bool(fonts) is not True");
 			if(_i2){
-				_o1 = PySequence_GetSlice(utext, i, j); if(!_o1) ERROR_EXIT();
-				_o2 = PyTuple_New(2); if(!_o2) ERROR_EXIT();
+				_o1 = PySequence_GetSlice(utext, i, j); if(!_o1) EXC_EXIT(PyExc_IndexError,"utext[%d:%d] failed",i,j);
+				_o2 = PyTuple_New(2); if(!_o2) EXC_EXIT(PyExc_MemoryError,"create tuple of length 2 failed");
 				PyTuple_SET_ITEM(_o2, 0, _o1);
 				Py_INCREF(fonts);
 				PyTuple_SET_ITEM(_o2, 1, fonts);
-				_o1 = unicode2T1(module,_o2,NULL); if(!_o1) ERROR_EXIT();
+				_o1 = unicode2T1(module,_o2,NULL); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"PyTuple_SET_ITEM(_o2,1,fonts) failed");
 				Py_DECREF(_o2); _o2 = 0;
-				_o3 = PyTuple_New(1); if(!_o3) ERROR_EXIT();
+				_o3 = PyTuple_New(1); if(!_o3) EXC_EXIT(PyExc_MemoryError,"create tuple of length 1 failed");
 				PyTuple_SET_ITEM(_o3, 0, _o1);
-				_o1 = _GetAttrString(R, "extend"); if(!_o1) ERROR_EXIT();
-				_o2 = PyObject_CallObject(_o1, _o3); if(!_o2) ERROR_EXIT();
+				_o1 = _GetAttrString(R, "extend"); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"no attriute extend");
+				_o2 = PyObject_CallObject(_o1, _o3); if(!_o2) EXC_EXIT(PyExc_TypeError,"result.extend call failed");
 				Py_DECREF(_o1);
 				Py_DECREF(_o3);
 				Py_DECREF(_o2); _o1 = _o2 = _o3 = NULL;
 				}
 			else{
 				_o3 = _GetAttrString(font,"_notdefChar");
-				if(!_o3) ERROR_EXIT();
-				_o2 = PyLong_FromLong((j - i)); if(!_o2) ERROR_EXIT();
-				_o1 = PyNumber_Multiply(_o3, _o2); if(!_o1) ERROR_EXIT();
+				if(!_o3) EXC_EXIT(PyExc_RuntimeError,"missing _notdefChar");
+				_o2 = PyLong_FromLong((j - i)); if(!_o2) EXC_EXIT(PyExc_ValueError,"int((%d - %d)) failed",j,i);
+				_o1 = PyNumber_Multiply(_o3, _o2); if(!_o1) EXC_EXIT(PyExc_ArithmeticError,"_notdefChar multiply failed");
 				Py_DECREF(_o2); Py_DECREF(_o3); _o2=_o3=NULL;
-				_o2 = PyTuple_New(2); if(!_o2) ERROR_EXIT();
-				_o3 = _GetAttrString(font,"_notdefFont"); if(!_o3) ERROR_EXIT();
+				_o2 = PyTuple_New(2); if(!_o2) EXC_EXIT(PyExc_MemoryError,"create tuple of length 2 failed");
+				_o3 = _GetAttrString(font,"_notdefFont"); if(!_o3) EXC_EXIT(PyExc_AttributeError,"missing _notdefFont");
 				PyTuple_SET_ITEM(_o2, 0, _o3);
 				PyTuple_SET_ITEM(_o2, 1, _o1);
 				Py_INCREF(_o3); _o3=NULL;
 				_o1 = NULL;
-				if(PyList_Append(R, _o2)) ERROR_EXIT();
+				if(PyList_Append(R, _o2)) EXC_EXIT(PyExc_RuntimeError,"Could not append to result");
 				Py_DECREF(_o2); _o2 = NULL;
 				}
 
-			_o1 = PySequence_GetSlice(utext, j, 0x7fffffff); if(!_o1) ERROR_EXIT();
+			_o1 = PySequence_GetSlice(utext, j, 0x7fffffff); if(!_o1) EXC_EXIT(PyExc_IndexError,"utext[%d:] failed",j);
 			Py_DECREF(utext);
 			utext = _o1;
 			_o1 = NULL;
 			}
 		}
-	if(_i1<0) ERROR_EXIT();
+	if(_i1<0) EXC_EXIT(PyExc_ValueError,"_i1=%d became negative",_i1);
 
 	Py_INCREF(R);
 	res = R;
 	goto L_OK;
 
-L_ERR:
-	ADD_TB(module,"unicode2T1");
+L_exit:
 	Py_XDECREF(_o1);
 	Py_XDECREF(_o2);
 	Py_XDECREF(_o3);
@@ -767,36 +734,35 @@ static PyObject *instanceStringWidthT1(PyObject *module, PyObject *args, PyObjec
 	if(!encoding) encStr="utf8";
 	else {
 		encObj = _GetStringBuf(encoding, &encStr);
-		if(!encObj) ERROR_EXIT();
+		if(!encObj) EXC_EXIT(PyExc_ValueError,"cannot obtain encoding");
 		}
 
 	if(!PyUnicode_Check(text)){
 		if(PyBytes_Check(text)){
 			_o1=PyUnicode_Decode(PyBytes_AS_STRING(text), PyBytes_GET_SIZE(text), encStr,"strict");
-			if(!_o1) ERROR_EXIT();
+			if(!_o1) EXC_EXIT(PyExc_ValueError,"cannot decode text argument");
 			Py_DECREF(text);
 			text = _o1;
 			_o1 = NULL;
 			}
 		else{
-			PyErr_SetString(PyExc_ValueError, "invalid type for argument text");
-			ERROR_EXIT();
+			EXC_EXIT(PyExc_ValueError, "invalid type for argument text");
 			}
 		}
 
-	_o3 = PyList_New(1); if(!_o3) ERROR_EXIT();
+	_o3 = PyList_New(1); if(!_o3) EXC_EXIT(PyExc_MemoryError,"PyList_New(1) failed");
 	Py_INCREF(self);
 	PyList_SET_ITEM(_o3, 0, self);
-	_o2 = _GetAttrString(self, "substitutionFonts"); if(!_o2) ERROR_EXIT();
-	_o1 = PyNumber_Add(_o3, _o2); if(!_o1) ERROR_EXIT();
+	_o2 = _GetAttrString(self, "substitutionFonts"); if(!_o2) EXC_EXIT(PyExc_RuntimeError,"missing substitutionFonts");
+	_o1 = PyNumber_Add(_o3, _o2); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"substitution addition failed");
 	Py_DECREF(_o3); _o3 = 0;
 	Py_DECREF(_o2); _o2 = NULL;
-	_o3 = PyTuple_New(2); if(!_o3) ERROR_EXIT();
+	_o3 = PyTuple_New(2); if(!_o3) EXC_EXIT(PyExc_MemoryError,"PyTuple_New(2) failed");
 	Py_INCREF(text);
 	PyTuple_SET_ITEM(_o3, 0, text);
 	PyTuple_SET_ITEM(_o3, 1, _o1);
 	_o1 = NULL;
-	_o2 = unicode2T1(module,_o3,NULL); if(!_o2) ERROR_EXIT();
+	_o2 = unicode2T1(module,_o3,NULL); if(!_o2) EXC_EXIT(PyExc_RuntimeError,"unicode2T1 call failed");
 	Py_DECREF(_o3); _o3 = NULL;
 	L = _o2;
 	_o2 = NULL;
@@ -804,20 +770,20 @@ static PyObject *instanceStringWidthT1(PyObject *module, PyObject *args, PyObjec
 	n = PyList_GET_SIZE(L);
 
 	for(s=i=0;i<n;++i){
-		_o1 = PyList_GetItem(L,i); if(!_o1) ERROR_EXIT();
+		_o1 = PyList_GetItem(L,i); if(!_o1) EXC_EXIT(PyExc_IndexError,"L[%d] failed",i);
 		Py_INCREF(_o1);
 
-		_o2 = PySequence_GetItem(_o1, 0); if(!_o2) ERROR_EXIT();
+		_o2 = PySequence_GetItem(_o1, 0); if(!_o2) EXC_EXIT(PyExc_IndexError,"L[%d][0] failed",i);
 		Py_XDECREF(f);
 		f = _o2;
 		_o2 = NULL;
 
-		_o2 = _GetAttrString(f, "widths"); if(!_o2) ERROR_EXIT();
+		_o2 = _GetAttrString(f, "widths"); if(!_o2) EXC_EXIT(PyExc_AttributeError,"no widths");
 		Py_DECREF(f);
 		f = _o2;
 		_o2 = NULL;
 
-		_o2 = PySequence_GetItem(_o1, 1); if(!_o2) ERROR_EXIT();
+		_o2 = PySequence_GetItem(_o1, 1); if(!_o2) EXC_EXIT(PyExc_IndexError,"L[%d][1] failed",i);
 		Py_XDECREF(t);
 		t = _o2;
 		Py_DECREF(_o1);
@@ -828,20 +794,19 @@ static PyObject *instanceStringWidthT1(PyObject *module, PyObject *args, PyObjec
 
 		for(j=0;j<m;++j){
 			_i1 = (long)(b[j]);
-			_o2 = PyList_GetItem(f,_i1); if(!_o2) {PyErr_Format(PyExc_IndexError,"widths index %d out of range",_i1);ERROR_EXIT();}
+			_o2 = PyList_GetItem(f,_i1); if(!_o2) {EXC_EXIT(PyExc_IndexError,"widths index %d out of range",_i1);}
 			_i1 = PyLong_AsLong(_o2);
 			_o2 = NULL;	/*we borrowed this*/
-			if(PyErr_Occurred()) ERROR_EXIT();
+			if(PyErr_Occurred()) EXC_EXIT(PyExc_RuntimeError,"longint conversion failed");
 			s += _i1;
 			}
 		}
 
-	_o1 = PyFloat_FromDouble((s * 0.001)); if(!_o1) ERROR_EXIT();
-	res = PyNumber_Multiply(_o1, size); if(!res) ERROR_EXIT();
+	_o1 = PyFloat_FromDouble((s * 0.001)); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"float(s*0.001) failed");
+	res = PyNumber_Multiply(_o1, size); if(!res) EXC_EXIT(PyExc_RuntimeError,"multiply by size failed");
 	Py_DECREF(_o1);
 	goto L_OK;
-L_ERR:
-	ADD_TB(module,"instanceStringWidthT1");
+L_exit:
 	Py_XDECREF(_o1);
 	Py_XDECREF(_o2);
 	Py_XDECREF(_o3);
@@ -871,22 +836,22 @@ static PyObject *instanceStringWidthTTF(PyObject *module, PyObject *args, PyObje
 		Py_INCREF(encoding);
 		}
 	else{
-		_o1 = PyUnicode_FromString("utf8"); if(!_o1) ERROR_EXIT();
+		_o1 = PyUnicode_FromString("utf8"); if(!_o1) EXC_EXIT(PyExc_UnicodeDecodeError,"utf8 decode failed");
 		encoding = _o1;
 		_o1 = NULL;
 		}
 
 	if(!PyUnicode_Check(text)){
-		i = PyObject_IsTrue(encoding); if(i<0) ERROR_EXIT();
+		i = PyObject_IsTrue(encoding); if(i<0) EXC_EXIT(PyExc_RuntimeError,"truth(encoding)=%d",i);
 		if(!i){
 			Py_DECREF(encoding);
-			encoding = PyUnicode_FromString("utf8"); if(!encoding) ERROR_EXIT();
+			encoding = PyUnicode_FromString("utf8"); if(!encoding) EXC_EXIT(PyExc_UnicodeDecodeError,"utf8 decode failed");
 			}
-		_o1 = _GetAttrString(text, "decode"); if(!_o1) ERROR_EXIT();
-		_o3 = PyTuple_New(1); if(!_o3) ERROR_EXIT();
+		_o1 = _GetAttrString(text, "decode"); if(!_o1) EXC_EXIT(PyExc_AttributeError,"missing attribute decode");
+		_o3 = PyTuple_New(1); if(!_o3) EXC_EXIT(PyExc_MemoryError,"PyTuple_New(1) failed");
 		Py_INCREF(encoding);
 		PyTuple_SET_ITEM(_o3, 0, encoding);
-		_o2 = PyObject_CallObject(_o1, _o3); if(!_o2) ERROR_EXIT();
+		_o2 = PyObject_CallObject(_o1, _o3); if(!_o2) EXC_EXIT(PyExc_RuntimeError,"call of decode attribute failed");
 		Py_DECREF(_o1);
 		Py_DECREF(_o3); _o1 = _o3 = NULL;
 		Py_DECREF(text);
@@ -894,36 +859,36 @@ static PyObject *instanceStringWidthTTF(PyObject *module, PyObject *args, PyObje
 		}
 
 	/*self.face.charWidths --> _o1, self.face.defaultWidth --> _o3*/
-	_o2 = _GetAttrString(self, "face"); if(!_o2) ERROR_EXIT();
-	_o1 = _GetAttrString(_o2, "charWidths"); if(!_o1) ERROR_EXIT(); if(!PyDict_Check(_o1)){PyErr_SetString(PyExc_TypeError, "TTFontFace instance charWidths is not a dict");ERROR_EXIT();}
-	_o3 = _GetAttrString(_o2, "defaultWidth"); if(!_o3) ERROR_EXIT();
+	_o2 = _GetAttrString(self, "face"); if(!_o2) EXC_EXIT(PyExc_AttributeError,"missing attribute face");
+	_o1 = _GetAttrString(_o2, "charWidths"); if(!_o1) EXC_EXIT(PyExc_AttributeError,"no attribute charWidths");
+	if(!PyDict_Check(_o1)){EXC_EXIT(PyExc_TypeError, "TTFontFace instance charWidths is not a dict");}
+	_o3 = _GetAttrString(_o2, "defaultWidth"); if(!_o3) EXC_EXIT(PyExc_AttributeError,"missing attribute defaultWidth");
 	Py_DECREF(_o2); _o2 = NULL;
 	dw = PyFloat_AsDouble(_o3);
-	if(PyErr_Occurred()) ERROR_EXIT();
+	if(PyErr_Occurred()) EXC_EXIT(PyExc_RuntimeError,"float() failed");
 	Py_DECREF(_o3);	_o3=NULL;
 
 	n = PyUnicode_GET_SIZE(text);
 	b = PyUnicode_AS_UNICODE(text);
 
 	for(s=i=0;i<n;++i){
-		_o3 = PyLong_FromLong((long)b[i]); if(!_o3) ERROR_EXIT();
+		_o3 = PyLong_FromLong((long)b[i]); if(!_o3) EXC_EXIT(PyExc_RuntimeError,"FromLong failed");
 		_o2 = PyDict_GetItem(_o1,_o3);
 		Py_DECREF(_o3); _o3 = NULL;
 		if(!_o2) _d1 = dw;
 		else{
 			_d1 = PyFloat_AsDouble(_o2);
 			_o2=NULL;	/*no decref as we borrowed it*/
-			if(PyErr_Occurred()) ERROR_EXIT();
+			if(PyErr_Occurred()) EXC_EXIT(PyExc_RuntimeError,"float-->double failed");
 			}
 		s += _d1;
 		}
 	Py_DECREF(_o1);
-	_o1 = PyFloat_FromDouble((s * 0.001)); if(!_o1) ERROR_EXIT();
-	res = PyNumber_Multiply(_o1, size); if(!res) ERROR_EXIT();
+	_o1 = PyFloat_FromDouble((s * 0.001)); if(!_o1) EXC_EXIT(PyExc_RuntimeError,"float(s*0.001) failed");
+	res = PyNumber_Multiply(_o1, size); if(!res) EXC_EXIT(PyExc_RuntimeError,"multiply by size failed");
 	Py_DECREF(_o1);
 	goto L_OK;
-L_ERR:
-	ADD_TB(module,"instanceStringWidthTTF");
+L_exit:
 	Py_XDECREF(_o1);
 	Py_XDECREF(_o2);
 	Py_XDECREF(_o3);
@@ -980,7 +945,7 @@ static int Box_set_character(BoxObject *self, PyObject *value)
 		char *v = PyBytes_AsString(value);
 		if(!v) return -1;
 		if(PyBytes_GET_SIZE(value)!=1){
-			PyErr_Format(PyExc_AttributeError,"Bad size %d('%s') for attribute character",PyBytes_GET_SIZE(value),v);
+			EXC_SET(PyExc_AttributeError,"Bad size %d('%s') for attribute character",PyBytes_GET_SIZE(value),v);
 			return -1;
 			}
 		self->character = v[0];
@@ -1002,8 +967,8 @@ static int Box_setattr(BoxObject *self, char *name, PyObject* value)
 			!strcmp(name,"is_penalty") ||
 			!strcmp(name,"is_box") ||
 			!strcmp(name,"is_glue")
-			) PyErr_Format(PyExc_AttributeError, "readonly attribute %s", name);
-	else PyErr_Format(PyExc_AttributeError, "no attribute %s", name);
+			) EXC_SET(PyExc_AttributeError, "readonly attribute %s", name);
+	else EXC_SET(PyExc_AttributeError, "no attribute %s", name);
 	return -1;
 }
 
@@ -1261,6 +1226,8 @@ static PyTypeObject BoxList_type = {
 	"\tGlue(width,stretch,shrink) creates a Knuth glue Box with the specified width, stretch and shrink.\n" \
 	"\tPenalty(width,penalty,flagged=0) creates a Knuth penalty Box with the specified width and penalty.\n" \
 	"\tBoxList() creates a knuth box list.\n"
+#else
+#define _BOX__DOC__
 #endif
 
 PyDoc_STRVAR(__DOC__,
@@ -1302,12 +1269,11 @@ static struct PyMethodDef _methods[] = {
 	};
 
 /*Initialization function for the module*/
-#ifdef isPy3
 static struct PyModuleDef moduledef = {
 	PyModuleDef_HEAD_INIT,
 	"_rl_accel",
 	__DOC__,
-	sizeof(struct module_state),
+	MODULE_STATE_SIZE,
 	_methods,
 	NULL,
 	NULL,
@@ -1316,30 +1282,17 @@ static struct PyModuleDef moduledef = {
 	};
 
 PyMODINIT_FUNC PyInit__rl_accel(void)
-#else
-void init_rl_accel(void)
-#endif
 {
 	PyObject			*module=NULL, *moduleVersion=NULL;
 	/*Create the module and add the functions and module doc string*/
-#ifdef isPy3
 	module = PyModule_Create(&moduledef);
-#else
-	module = Py_InitModule3("_rl_accel", _methods,__DOC__);
-#endif
 	if(!module) goto err;
 	/*Add some symbolic constants to the module */
-	moduleVersion = PyBytes_FromString(VERSION);
+	moduleVersion = PyUnicode_FromString(VERSION);
 	if(!moduleVersion)goto err;
-#ifndef isPy3
-	GETSTATE(module)->module = module;
-#endif
 	PyModule_AddObject(module, "version", moduleVersion);
 
 #ifdef	HAVE_BOX
-#ifndef isPy3
-	BoxType.ob_type = &PyType_Type;
-#endif
 	if(PyType_Ready(&BoxType)<0) goto err;
 	BoxList_type.tp_base = &PyList_Type;
 	if(PyType_Ready(&BoxList_type)<0) goto err;
@@ -1347,18 +1300,10 @@ void init_rl_accel(void)
 	if(PyModule_AddObject(module, "BoxList", (PyObject *)&BoxList_type)<0)goto err;
 #endif
 
-#ifdef isPy3
 	return module;
-#else
-	return;
-#endif
 
 err:/*Check for errors*/
-#ifdef isPy3
 	Py_XDECREF(moduleVersion);
 	Py_XDECREF(module);
 	return NULL;
-#else
-	if (PyErr_Occurred()) Py_FatalError("can't initialize module _rl_accel");
-#endif
 }
